import inspect
from itertools import islice

from pygame import Surface
from typing import List

from .CacheComponent import CacheComponent
from .Component import Component
from .IComponentListener import IComponentListener
from ..IUIEventHandler import IUIEventHandler
from ..theme.Theme import Theme


class CompositeComponent(Component, IComponentListener, IUIEventHandler):

    def __init__(self, theme: Theme):
        super().__init__(theme)
        self.__components: List[Component] = []

    def addComponent(self, component: Component, cache: bool = False):
        if cache:
            component = CacheComponent(component)
        self.__components.append(component)

    def deleteComponents(self):
        for component in self.__components:
            component.dispose()
        self.__components.clear()

    # Component interface

    def dispose(self):
        for component in self.__components:
            component.dispose()

    def needRefresh(self) -> bool:
        for component in self.__components:
            if component.needRefresh:
                return True
        return False

    def render(self, surface: Surface):
        for component in self.__components:
            component.render(surface)


# Dynamic creation of UI Event Handler methods
eventHandlerMethods = inspect.getmembers(IUIEventHandler, predicate=inspect.isfunction)
for name, method in eventHandlerMethods:
    if name.startswith("__"):
        continue
    signature = inspect.signature(method)
    functionArguments = ", ".join(signature.parameters)
    methodArguments = ", ".join(islice(signature.parameters, 1, None))
    source = f'''
def CompositeComponent_{name}({functionArguments}):
    for component in reversed(self._CompositeComponent__components):
        if isinstance(component, IUIEventHandler):
            if component.{name}({methodArguments}):
                return True
    return False
    '''
    exec(source)
    setattr(CompositeComponent, name, globals()['CompositeComponent_' + name])

# Dynamic creation of Component listener methods
listenerMethods = inspect.getmembers(IComponentListener, predicate=inspect.isfunction)
for name, method in listenerMethods:
    if name.startswith("__"):
        continue
    signature = inspect.signature(method)
    functionArguments = ", ".join(signature.parameters)
    methodArguments = ", ".join(islice(signature.parameters, 1, None))
    source = f'''
def CompositeComponent_{name}({functionArguments}):
    for component in reversed(self._CompositeComponent__components):
        if isinstance(component, IComponentListener):
            component.{name}({methodArguments})
    '''
    exec(source)
    setattr(CompositeComponent, name, globals()['CompositeComponent_' + name])


