from typing import Tuple, Optional

from core.state import World
from tools.vector import vectorDivI
from .LayerComponent import LayerComponent
from ..CompositeComponent import CompositeComponent
from ...IUIEventHandler import IUIEventHandler
from ...Mouse import Mouse
from ...theme.Theme import Theme


class WorldComponent(CompositeComponent, IUIEventHandler):

    def __init__(self, theme: Theme, world: World):
        super().__init__(theme)
        self.__world = world
        self.__previousCell: Optional[Tuple[int, int]] = None
        self.__mouseButtonDown = False

        for name in world.layerNames:
            layer = LayerComponent(theme, world, name)
            self.addComponent(layer, cache=True)

        self.__tileSize = theme.getTileset(world.layerNames[0]).tileSize

    # UI Event Handler

    def __computeCellCoordintates(self, pixel: Tuple[int, int]) -> Optional[Tuple[int, int]]:
        coords = vectorDivI(pixel, self.__tileSize)
        if not self.__world.contains(coords):
            return None
        return coords

    def mouseButtonDown(self, mouse: Mouse):
        super().mouseButtonDown(mouse)
        cell = self.__computeCellCoordintates(mouse.pixel)
        if cell is None:
            return False
        self.__mouseButtonDown = True
        self.notifyWorldCellClicked(cell, mouse)
        return True

    def mouseMove(self, mouse: Mouse):
        cell = self.__computeCellCoordintates(mouse.pixel)
        if cell is None:
            self.__previousCell = None
            return False
        if self.__previousCell is not None and cell == self.__previousCell:
            return False
        self.__previousCell = cell
        self.notifyWorldCellEntered(cell, mouse, self.__mouseButtonDown)
        return True

    def mouseButtonUp(self, mouse: Mouse):
        self.__mouseButtonDown = False
        return True

    def mouseEnter(self, mouse: Mouse):
        self.__mouseButtonDown = False
        return True

    def mouseLeave(self):
        self.__mouseButtonDown = False
        return True
