import random
from typing import Tuple

import pygame
from pygame import Surface

from core.constants import CellValue, CellValueRanges
from core.logic import Logic
from core.state import World
from .GameMode import GameMode
from ..Mouse import Mouse
from ..component.IComponentListener import IComponentListener
from ..component.world.WorldComponent import WorldComponent
from ..theme.Theme import Theme


class EditGameMode(GameMode, IComponentListener):

    def __init__(self, theme: Theme, world: World):
        super().__init__(theme)
        self.__world = world
        self.__logic = Logic(world)
        self.__font = theme.getFont("default")

        # Components
        self.__worldComponent = WorldComponent(theme, world)
        self.addComponent(self.__worldComponent)

        # Brush
        self.__brushLayer = "ground"

        # Listener
        self.__worldComponent.registerListener(self)

    # Game loop

    def dispose(self):
        self.__worldComponent.removeListener(self)
        super().dispose()

    def update(self):
        self.__logic.executeCommands()

    def render(self, surface: Surface):
        super().render(surface)

        color = self.theme.getFontColor("default")
        textSurfaces = [
            self.__font.render(message, False, color)
            for message in [
                f"Current brush: {self.__brushLayer}",
                "Click left: add; middle: fill; right: remove",
                "F1/F2/F3: ground/impassable/objects"
            ]
        ]
        y = surface.get_height()
        for textSurface in reversed(textSurfaces):
            y -= textSurface.get_height() + 1
            surface.blit(textSurface, (0, y))

    # UI Event handler

    def keyDown(self, key: int):
        if key == pygame.K_F1:
            self.__brushLayer = "ground"
            return True
        elif key == pygame.K_F2:
            self.__brushLayer = "impassable"
            return True
        elif key == pygame.K_F3:
            self.__brushLayer = "objects"
            return True
        else:
            return super().keyDown(key)

    # Component listener

    def __updateCell(self, cell: Tuple[int, int], mouse: Mouse):
        if mouse.button1 or mouse.button2:
            if self.__brushLayer == "ground":
                value = CellValue.GROUND_EARTH
            else:
                minValue = CellValueRanges[self.__brushLayer][0]
                maxValue = CellValueRanges[self.__brushLayer][1]
                value = CellValue(random.randint(minValue, maxValue - 1))
            fill = mouse.button2
        elif mouse.button3:
            if self.__brushLayer == "ground":
                value = CellValue.GROUND_SEA
            else:
                value = CellValue.NONE
            fill = False
        else:
            return
        Command = self.__logic.getSetLayerValueCommand(self.__brushLayer)
        command = Command(cell, value, fill)
        self.__logic.addCommand(command)

    def worldCellClicked(self, cell: Tuple[int, int], mouse: Mouse):
        self.__updateCell(cell, mouse)

    def worldCellEntered(self, cell: Tuple[int, int], mouse: Mouse, dragging: bool):
        if dragging:
            self.__updateCell(cell, mouse)
