import os
from typing import Dict, Tuple, Union, Optional

import pygame
from pygame.font import Font
from pygame.surface import Surface

from core.constants import CellValue
from .Tileset import Tileset


class Theme:
    def __init__(self):
        # Tilesets
        self.__surfaces: Dict[str, Surface] = {}
        self.__tilesets: Dict[str, Tileset] = {}
        for name, tilesDef in tilesDefs.items():
            tileset = Tileset(self, tilesDef["tileSize"], tilesDef["imageFile"])
            for value, coords in tilesDef["tiles"].items():
                tileset.addTile(value, coords)
            self.__tilesets[name] = tileset

        # Text
        self.__fontsDef = {
            "default": {
                "file": "font/prstart/prstartk.ttf",
                "size": 8
            }
        }
        self.__fontColors = {
            "default": (100, 75, 50)
        }
        self.__fonts = {}

    def getSurface(self, imageFile: str) -> Surface:
        if imageFile not in self.__surfaces:
            fullPath = os.path.join("assets", imageFile)
            if not os.path.exists(fullPath):
                raise ValueError(f"No file '{fullPath}'")
            self.__surfaces[imageFile] = pygame.image.load(fullPath).convert_alpha()
        return self.__surfaces[imageFile]

    def getTileset(self, name: str) -> Tileset:
        if name not in self.__tilesets:
            raise ValueError(f"No tileset {name}")
        return self.__tilesets[name]

    def getFont(self, name: Optional[str] = None) -> Font:
        if name is None:
            name = "default"
        if name not in self.__fontsDef:
            raise ValueError("No font {}".format(name))
        fontDef = self.__fontsDef[name]
        file = os.path.join("assets", fontDef["file"])
        size = fontDef["size"]
        fontId = file, size
        if fontId not in self.__fonts:
            self.__fonts[fontId] = pygame.font.Font(file, size)
        return self.__fonts[fontId]

    def getFontColor(self, name: Union[None, str] = None) -> Tuple[int, int, int]:
        if name is None:
            name = "default"
        if name not in self.__fontColors:
            raise ValueError("No font color {}".format(name))
        return self.__fontColors[name]


tilesDefs = {
    "ground": {
        "imageFile": "toen/ground.png",
        "tileSize": (16, 16),
        "tiles": {
            CellValue.GROUND_SEA: (4, 7),
            CellValue.GROUND_EARTH: (2, 7),
        }
    },
    "impassable": {
        "imageFile": "toen/impassable.png",
        "tileSize": (16, 16),
        "tiles": {
            CellValue.IMPASSABLE_RIVER: (0, 1),
            CellValue.IMPASSABLE_POND: (1, 0),
            CellValue.IMPASSABLE_MOUNTAIN: (4, 0),
        }
    },
    "objects": {
        "imageFile": "toen/objects.png",
        "tileSize": (16, 16),
        "tiles": {
            CellValue.OBJECTS_SIGN: (1, 0),
            CellValue.OBJECTS_HILL: (4, 0),
            CellValue.OBJECTS_ROCKS: (6, 1),
            CellValue.OBJECTS_TREES: (5, 0),
            CellValue.OBJECTS_MILL: (0, 1),
            CellValue.OBJECTS_HOUSES: (1, 2),
            CellValue.OBJECTS_ROAD_DIRT: (0, 3),
            CellValue.OBJECTS_ROAD_STONE: (4, 3),
            CellValue.OBJECTS_FARM: (8, 0),
            CellValue.OBJECTS_CAMP: (8, 1),
        },
    },
}
